@@ -19,10 +19,10 @@ from urllib.parse import quote
 
 import httpx
 
-from .vendor.myutils import at_dir
+from .vendor.myutils import at_dir, file_lock
 from .vendor.htmlutils import parse_document_from_httpx
 
-from .cmd import git_pull, git_push, UNTRUSTED_PREFIX
+from .cmd import git_push, git_pull, UNTRUSTED_PREFIX
 from .cmd import run_cmd as _run_cmd
 from . import const, intl
 from .const import _G, SPECIAL_FILES
@@ -32,17 +32,16 @@ from . import mediawiki2pkgbuild
 from .pkgbuild import get_srcinfo
 
 git_push
+git_pull
 
 logger = logging.getLogger(__name__)
 
 _g = SimpleNamespace()
-UserAgent = 'lilac/0.2b (package auto-build bot, by lilydjwg)'
 
 logging.getLogger('httpcore').setLevel(logging.ERROR)
 logging.getLogger('hpack').setLevel(logging.ERROR)
 
 s = httpx.Client(http2=True)
-s.headers['User-Agent'] = UserAgent
 
 VCS_SUFFIXES = ('-git', '-hg', '-svn', '-bzr')
 AUR_BLACKLIST = {
@@ -363,20 +362,30 @@ def _aur_exists(pkgbase: str) -> bool:
     r.raise_for_status()
   return r.status_code != 404
 
+def _ensure_aur_repo(pkgbase: str) -> Path:
+  lockfile = const.AUR_REPO_DIR / (pkgbase + '.lock')
+  env = os.environ.copy()
+  env['GIT_SSH_COMMAND'] = 'ssh -o ControlPersist=no'
+  with file_lock(lockfile):
+    aurpath = const.AUR_REPO_DIR / pkgbase
+    if not aurpath.is_dir():
+      logger.info('cloning AUR repo: %s', aurpath)
+      with at_dir(const.AUR_REPO_DIR):
+        _run_cmd(['git', 'clone', f'aur@aur.archlinux.org:{pkgbase}.git'],
+                env=env)
+    else:
+      with at_dir(aurpath):
+        # reset everything, dropping local commits
+        _run_cmd(['git', 'reset', '--hard', 'origin/master'])
+        _run_cmd(['git', 'pull', '--no-edit'], env=env)
+
+  return aurpath
+
 def _update_aur_repo_real(pkgbase: str) -> None:
   if not _aur_exists(pkgbase):
     raise LookupError('AUR package not exists, not updating!', pkgbase)
 
-  aurpath = const.AUR_REPO_DIR / pkgbase
-  if not aurpath.is_dir():
-    logger.info('cloning AUR repo: %s', aurpath)
-    with at_dir(const.AUR_REPO_DIR):
-      _run_cmd(['git', 'clone', f'aur@aur.archlinux.org:{pkgbase}.git'])
-  else:
-    with at_dir(aurpath):
-      # reset everything, dropping local commits
-      _run_cmd(['git', 'reset', '--hard', 'origin/master'])
-      git_pull()
+  aurpath = _ensure_aur_repo(pkgbase)
 
   with at_dir(aurpath):
     oldfiles = set(_run_cmd(['git', 'ls-files']).splitlines())
@@ -412,7 +421,9 @@ def _update_aur_repo_real(pkgbase: str) -> None:
     if p.returncode != 0:
       msg = f'[lilac] updated to {_G.built_version}'
       _run_cmd(['git', 'commit', '--no-gpg-sign', '-m', msg])
-      _run_cmd(['git', 'push'])
+      env = os.environ.copy()
+      env['GIT_SSH_COMMAND'] = 'ssh -o ControlPersist=no'
+      _run_cmd(['git', 'push'], env=env)
 
 def update_aur_repo() -> None:
   '''update the package on AUR if suitable.
@@ -426,8 +437,7 @@ def update_aur_repo() -> None:
     _update_aur_repo_real(pkgbase)
   except Exception as e:
     l10n = intl.get_l10n('mail')
-    _G.repo.send_error_report(
-      _G.mod,
+    _G.add_report(
       exc = e,
       subject = l10n.format_value('aur-submit-error'),
     )
@@ -469,33 +479,17 @@ def clean_directory() -> List[str]:
       os.unlink(f)
   return ret
 
-def _try_aur_url(name: str) -> bytes:
-  aur4url = 'https://aur.archlinux.org/cgit/aur.git/snapshot/{name}.tar.gz'
-  templates = [aur4url]
-  urls = [url.format(first_two=name[:2], name=name) for url in templates]
-  for url in urls:
-    for _ in range(2):
-      try:
-        response = s.get(url)
-      except httpx.ReadTimeout:
-        logger.warning('AUR download timed out.')
-        continue
-
-      if response.status_code == 200:
-        logger.debug("downloaded aur tarball '%s' from url '%s'", name, url)
-        return response.content
-      elif response.status_code >= 500:
-        logger.warning('AUR download failed.')
-      else:
-        break
-  logger.error("failed to find aur url for '%s'", name)
-  raise AurDownloadError(name)
+def _get_aur_tarball(name: str) -> bytes:
+  aurpath = _ensure_aur_repo(name)
+  return subprocess.check_output([
+    'git', 'archive', f'--prefix={name}/', 'HEAD',
+  ], cwd=aurpath)
 
 def _download_aur_pkgbuild(name: str) -> List[str]:
-  content = io.BytesIO(_try_aur_url(name))
+  content = io.BytesIO(_get_aur_tarball(name))
   files = []
   with tarfile.open(
-    name=name+".tar.gz", mode="r:gz", fileobj=content
+    name=name+".tar", mode="r:", fileobj=content
   ) as tarf:
     for tarinfo in tarf:
       try:
@@ -554,7 +548,10 @@ def aur_pre_build(
 
   pkgver, pkgrel = get_pkgver_and_pkgrel()
   _g.aur_pre_files = clean_directory()
-  _g.aur_building_files = _download_aur_pkgbuild(name)
+  try:
+    _g.aur_building_files = _download_aur_pkgbuild(name)
+  except Exception:
+    raise AurDownloadError(name)
 
   aur_pkgver, aur_pkgrel = get_pkgver_and_pkgrel()
   if pkgver and pkgver == aur_pkgver:
